/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.DisplayBean.NodeShape;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.geom.Rectangle2D;

/**
 * Renders a {@link RectangleNodeShape} with a small {@link TriangleNodeShape} drawn
 * in the bottom middle of the node.
 * 
 * @author Chris Callendar
 */
public class RectangleTriangleNodeShape extends RectangleNodeShape {
	
	public static final String NODE_SHAPE = "Rectangle (Triangle Overlay)";
	
	private final TriangleNodeShape triangle = new TriangleNodeShape();

	public RectangleTriangleNodeShape() {
		this(NODE_SHAPE);
	}
	
	public RectangleTriangleNodeShape(String name) {
		super(name);
	}
	
	/** 
	 * @see ca.uvic.csr.shrimp.DisplayBean.NodeShape.AbstractNodeShape#clone()
	 */
	public Object clone() {
		NodeShape shape = new RectangleTriangleNodeShape();
		return shape;
	}
		
	/**
	 * @see ca.uvic.csr.shrimp.DisplayBean.NodeShape.NodeShape#getInnerBounds(java.awt.geom.Rectangle2D)
	 */
	public Rectangle2D.Double getInnerBounds(Rectangle2D outerBounds) {
		double x = outerBounds.getX() + 4;
		double y = outerBounds.getY() + 4;
		double w = outerBounds.getWidth() - 8;
		double space = outerBounds.getWidth() / 24;
		double h = outerBounds.getHeight() - 8 - space;
		return new Rectangle2D.Double(x, y, w, h);
	}
	
	protected Rectangle2D adjustHeight(Rectangle2D bounds) {
		double space = bounds.getWidth() / 10;
		return new Rectangle2D.Double(bounds.getX(), bounds.getY(), bounds.getWidth(), bounds.getHeight() - space);
	}
	
	protected void renderShape(Rectangle2D outerBounds, Graphics2D g2) {
		super.renderShape(adjustHeight(outerBounds), g2);
	}
	
	protected void renderBorder(Rectangle2D outerBounds, Graphics2D g2, float strokeWidth) {
		super.renderBorder(adjustHeight(outerBounds), g2, strokeWidth);
	}
	
	protected void renderImage(Rectangle2D outerBounds, Graphics2D g2, Image image, String imageSizingMode) {
		super.renderImage(adjustHeight(outerBounds), g2, image, imageSizingMode);
	}
	
	protected void renderOverlay(Rectangle2D bounds, Graphics2D g2, Color fillColor, Color strokeColor, float strokeWidth, NodeImage nodeImage) {
		double size = bounds.getWidth() / 5;
		double x = Math.max(0, bounds.getCenterX() - (size / 2));
		double y = Math.max(0, bounds.getMaxY() - size);

		// Render the small TriangleNodeShape at the bottom of the bounds
		Rectangle2D triangleBounds = new Rectangle2D.Double(x, y, size, size);
		Color triangleColor = fillColor.darker(); //GraphicsUtils.invertColor(fillColor);
		triangle.render(triangleBounds, g2, triangleColor, strokeColor, 1f, nodeImage);
	}
}
