/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.jambalaya;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JOptionPane;

import ca.uvic.csr.shrimp.ShrimpConstants;
import ca.uvic.csr.shrimp.ShrimpApplication.AbstractShrimpApplication;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.gui.ActionManager.ShrimpAction;
import ca.uvic.csr.shrimp.resource.ResourceHandler;
import ca.uvic.csr.shrimp.util.ShowInBrowserAction;
import edu.stanford.smi.protege.model.Project;

/**
 * Implements Shrimp as a plugin for Protege, also known as "Jambalaya"
 * The JambalayaApplication has at most one JambalayaProject at a given time,
 * unlike stand-alone Shrimp which can have many.
 * 
 * @author Nasir Rather, Rob Lintern, Chris Callendar
 */
public class JambalayaApplication extends AbstractShrimpApplication {

	/**Title of the Jambalaya application. */
	public static final String APPLICATION_TITLE = "Jambalaya";

	/** Filename where properties for the Jambalaya application will be stored */
	private static final String PROPERTIES_FILE_NAME = "plugins/ca.uvic.cs.chisel.jambalaya/Jambalaya.properties";
		
	/** key for classes split pane location */
    public static final String PROPERTY_KEY__CLS_SPLIT_PANE_LOCATION = "Classes Divider Location"; 
    /** key for instances split pane location */
    public static final String PROPERTY_KEY__INSTANCE_SPLIT_PANE_LOCATION = "Instances Divider Location"; 
	/** default location for classes split pane */
    public final static int PROPERTY_DEFAULT__CLS_SPLIT_PANE_LOCATION = 150;
    /** default location for instances split pane */
	public final static int PROPERTY_DEFAULT__INSTANCE_SPLIT_PANE_LOCATION = 200;
    
    public final static String QUICK_VIEW_CLASS_TREE = "Class Tree";
    public final static String QUICK_VIEW_CLASS_INSTANCE_TREE = "Class & Instance Tree";
    public final static String QUICK_VIEW_CLASS_INDIVIDUAL_TREE = "Class & Individual Tree";
    public final static String QUICK_VIEW_DOMAIN_RANGE = "Domain/Range";
    public final static String QUICK_VIEW_PART_OF_TREE = "Part_of Tree";
    public final static String QUICK_VIEW_QUERY_VIEW = "Query View";
    
	/**
	 * Creates the Jambalaya application.
	 */
	public JambalayaApplication() {
        this(PROPERTIES_FILE_NAME);
	}
    
    public JambalayaApplication (String propertiesFilename) {
        super(propertiesFilename, APPLICATION_TITLE, 1);

        createApplicationActions();		// @tag Shrimp.createApplicationActions 
    }
    
	/**
	 * @see ca.uvic.csr.shrimp.ShrimpApplication.AbstractShrimpApplication#createMenus()
	 */
	public void createMenus() {
		JambalayaTab jambalayaTab = JambalayaProjectPlugin.getJambalayaTab();
		if (jambalayaTab != null) {
			jambalayaTab.createMenus();
		} else if (menuBar != null){
            super.createMenus();		    
        }
	}

	// Add actions for this application to the actionManager.
	protected void createApplicationSpecificActions() {
		super.createApplicationSpecificActions();
		
		//Help -> Online Manual 
		ShrimpAction action = new ShowInBrowserAction(ShrimpConstants.ACTION_NAME_ONLINE_MANUAL, ShrimpConstants.JAMBALAYA_MANUAL_WEBSITE, ResourceHandler.getIcon("icon_help.gif"));
		addApplicationAction(action, ShrimpConstants.MENU_HELP, ShrimpConstants.GROUP_A, 1);

		//Help -> Website
		action = new ShowInBrowserAction(ShrimpConstants.ACTION_NAME_JAMBALAYA_WEBSITE, ShrimpConstants.JAMBALAYA_WEBSITE, ResourceHandler.getIcon("icon_jambalaya.gif"));
		addApplicationAction(action, ShrimpConstants.MENU_HELP, ShrimpConstants.GROUP_A, 2);
	}
	
	public void close() {
		super.close();
	}

	public void fireApplicationActivatedEvent() {
		super.fireApplicationActivatedEvent();
	}
	
	public void fireApplicationDeactivatedEvent() {
		super.fireApplicationDeactivatedEvent();
	}
	
	public void fireApplicationStartedEvent() {
		super.fireApplicationStartedEvent();
	}
	
	/**
	 * Creates a Protege Project from the URI, and then creates the JambalayaProject for that project.
	 * @param sourceURI
	 * @return JambalayaProject or null
	 */
	public JambalayaProject createJambalayaProject(URI sourceURI) {
		JambalayaProject project = null;
		Project protegeProject = createProtegeProject(sourceURI);
		if (protegeProject != null) {
			project = createJambalayaProject(protegeProject, sourceURI);
		}
		return project;
	}
	
	
	public Project createProtegeProject(URI sourceURI) {
		Project protegeProject = null;
		if (sourceURI != null) {
            List errors = new ArrayList();
            protegeProject = new Project(sourceURI.toString(), errors);
            protegeProject.setDisplayHiddenFrames(false);
            if (!errors.isEmpty()) {
                String msg = "There were some errors loading the Protege project:";
                System.err.println(msg);
                System.err.println(errors);
				JOptionPane.showMessageDialog(getParentFrame(), msg + "\n" + errors, getName(), JOptionPane.ERROR_MESSAGE);
            }
		}
		return protegeProject;
	}

	/**
	 * Creates and returns a new JambalayaProject.
	 */
	public JambalayaProject createJambalayaProject(Project protegeProject, URI sourceURI) {
		// check if a project already exists for the given Protege project
		ShrimpProject[] projects = this.getProjects();
		for (int i = 0; i < projects.length; i++) {
			JambalayaProject jp = (JambalayaProject) projects[i];
			if (jp.getProtegeProject() == protegeProject) {
				System.out.println("Jambalaya Project already exists");
				return jp;
			}
		}
		
		JambalayaProject project = new JambalayaProject(this, protegeProject, sourceURI);
		addProject(project);
		
        return project;
	}
	
}