/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.jambalaya.ProtegeDataBean;

import java.awt.Component;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import javax.swing.Icon;

import ca.uvic.csr.shrimp.AttributeConstants;
import ca.uvic.csr.shrimp.DataBean.Artifact;
import ca.uvic.csr.shrimp.DataBean.Relationship;
import ca.uvic.csr.shrimp.gui.ProgressDialog;
import ca.uvic.csr.shrimp.util.CollectionUtils;
import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Frame;
import edu.stanford.smi.protege.model.Model;
import edu.stanford.smi.protege.model.Project;
import edu.stanford.smi.protege.model.Reference;
import edu.stanford.smi.protege.model.Slot;
import edu.stanford.smi.protegex.owl.model.OWLAnonymousClass;
import edu.stanford.smi.protegex.owl.model.OWLEnumeratedClass;
import edu.stanford.smi.protegex.owl.model.OWLHasValue;
import edu.stanford.smi.protegex.owl.model.OWLIntersectionClass;
import edu.stanford.smi.protegex.owl.model.OWLLogicalClass;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.OWLNAryLogicalClass;
import edu.stanford.smi.protegex.owl.model.OWLNamedClass;
import edu.stanford.smi.protegex.owl.model.OWLNames;
import edu.stanford.smi.protegex.owl.model.OWLQuantifierRestriction;
import edu.stanford.smi.protegex.owl.model.OWLRestriction;
import edu.stanford.smi.protegex.owl.model.ProtegeNames;
import edu.stanford.smi.protegex.owl.model.RDFIndividual;
import edu.stanford.smi.protegex.owl.model.RDFNames;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import edu.stanford.smi.protegex.owl.model.RDFResource;
import edu.stanford.smi.protegex.owl.model.RDFSClass;
import edu.stanford.smi.protegex.owl.model.RDFSNamedClass;
import edu.stanford.smi.protegex.owl.ui.icons.OverlayIcon;

/**
 * A quick hack to get some "OWLness" into Jambalaya.
 * The biggest issue here is how to turn logical expressions into artifact and relationships.
 * 
 * @author Rob Lintern
 */
public class ProtegeOWLDataBean extends ProtegeDataBean {
    private static final boolean DEFAULT_IMPORT_INHERITED_RELS = false;
    
    //private static final String SUFFIX_HAS_VALUE = " (Has Value)";
    //private static final String SUFFIX_QUANTIFIER_VALUE = " (Quantifier Value)";
    //private static final String SUFFIX_QUANTIFIER_CLS = " (Quantifier Cls)";  
    public final static String SUFFIX_NECESSARY = " (Necessary)";
    public final static String SUFFIX_NECESSARY_AND_SUFFICIENT = " (Necessary and Sufficient)";
    public final static String SUFFIX_CONNECTS_INDIVIDUALS = "";
    public final static String SUFFIX_DOMAIN_RANGE = " (Domain>Range)";
    //private final static String SUFFIX_RESTRICTION = " (Restriction)";
    public final static String SUFFIX_INHERITED = " (Inherited)";
	
    public final static String GROUP_NAME_CONNECTING_INDIVIDUALS = "Properties On Individuals";
    public final static String GROUP_NAME_PROPERTY_RESTRICTIONS = "Property Restrictions On Classes";
    public final static String GROUP_NAME_LOGICAL_DESCRIPTION = "Logical Descriptions";
    public final static String GROUP_NAME_DOMAIN_RANGE = "Domain>Range";
    public static final String GROUP_NAME_INHERITED = "Inherited";
	
    public final static String RESTRICTION_TYPE = "Restriction";
    public final static String LOGICAL_OPERATION_TYPE = "Logical Operation";
    public final static String PRIMITIVE_CLASS_TYPE = "Primitive Class";
    public final static String DEFINED_CLASS_TYPE = "Defined Class";
    public final static String INDIVIDUAL_TYPE = "Individual";
    public final static String ENUMERATION_CLASS_TYPE = "Enumeration";
    public final static String RDFS_CLASS_TYPE = "RDFS Class";

    private static final String REL_TYPE_NECESSARY_CONDITION_OF = "is a necessary condition of";
    //private static final String REL_TYPE_OPERATES_ON = "operatesOn";
	//private static final String REL_TYPE_EQUIVALENT_TO = "equivalent to";
	
	private static final String [] RDF_RDFS_OWL_SLOT_NAMES = {
	        OWLNames.Slot.DIFFERENT_FROM,
	        OWLNames.Slot.SAME_AS,
	        RDFNames.Slot.VALUE,
	        OWLNames.Slot.EQUIVALENT_CLASS,
	        OWLNames.Slot.ONE_OF,
	        OWLNames.Slot.DISJOINT_WITH,
	        OWLNames.Slot.DISTINCT_MEMBERS,
	        ProtegeNames.Slot.INFERRED_TYPE,
	        ProtegeNames.Slot.INFERRED_SUBCLASSES,
	        ProtegeNames.Slot.INFERRED_SUPERCLASSES,
	        /*TODO OWLNames.Slot.LOGICAL_OPERANDS */
	};
	
	
	private OWLModel owlModel;
    private Collection domainRangeRelsBuffer = null; // a collection of domain-->range relationships
	


	/**
	 * @param project
	 */
	public ProtegeOWLDataBean(Project project) {
		super(project);
		owlModel = (OWLModel) project.getKnowledgeBase();
	}
	
	/* (non-Javadoc)
     * @see ca.uvic.csr.shrimp.DataBean.ProtegeDataBean.ProtegeDataBean#clearBufferedData()
     */
    public void clearBufferedData() {
		if (dataIsDirty()) {
			super.clearBufferedData();
			domainRangeRelsBuffer = null;
		}
    }
	
	/**
	 * @param frame
	 */
	protected String frameToArtifactType(Frame frame) {
		String type = (String) frameToArtifactTypeMap.get (frame); 
		if (type == null) {
	        if (frame instanceof RDFSClass) {
	            if (frame instanceof OWLAnonymousClass) {
	                if (frame instanceof OWLRestriction) {
	                    type = RESTRICTION_TYPE;
	                } else if (frame instanceof OWLLogicalClass) {
	                    type = LOGICAL_OPERATION_TYPE;
	                } else if (frame instanceof OWLEnumeratedClass) {
	                    type = ENUMERATION_CLASS_TYPE;
	                }
	            } else if (frame instanceof RDFSNamedClass) {
	                if (frame instanceof OWLNamedClass) {
	                    if (((OWLNamedClass) frame).getDefinition() == null) {
	                        type = PRIMITIVE_CLASS_TYPE;
	                        //temporary for the NCI Thesaurus
	                        // give an artifact a type that represents the "kind" of
	                        // concept it is
                            // NOTE: traversing all the way up to the kind can be an expensive operation
	//                        Cls cls = (Cls) frame;
	//                        Collection rootClses = knowledgeBase.getRootCls().getDirectSubclasses();
	//                        if (!rootClses.isEmpty()) {
	//                            // if the first root cls ends with "_Kind" we'll
	//                            // make the assumption that
	//                            // we're dealing with the NCI Thesaurus
	//                            boolean kindFound = false;
	//                            Iterator iter = rootClses.iterator();
	//                            while (iter.hasNext() && !kindFound) {
	//                                Cls rootCls = (Cls) iter.next();
	//                                kindFound = getBrowserText(rootCls).indexOf("_Kind") != -1;
	//                            }
	//                            if (kindFound) {
	//                                if (rootClses.contains(cls)) {
	//                                    type = getBrowserText(cls);
	//                                } else {
	//                                    Collection superClses = cls.getSuperclasses();
	//                                    for (iter = superClses.iterator(); iter.hasNext();) {
	//                                        Cls superCls = (Cls) iter.next();
	//                                        if (rootClses.contains(superCls) && getBrowserText(superCls).indexOf("_Kind") != -1) {
	//                                            type = getBrowserText(superCls);
	//                                            break;
	//                                        }
	//                                    }
	//                                }
	//                            }
	//                        }
	                    } else {
	                        type = DEFINED_CLASS_TYPE;
	                    }
	                } else {
	                    type = RDFS_CLASS_TYPE;
	                }
	            }
	        } else if (frame instanceof RDFIndividual) {
	            type = INDIVIDUAL_TYPE;
	        } else {
	            type = super.frameToArtifactType(frame);
	        }
	        frameToArtifactTypeMap.put(frame, type);
		}
        return type;
    }
	
	/* (non-Javadoc)
     * @see ca.uvic.csr.shrimp.DataBean.ProtegeDataBean.ProtegeDataBean#createArtifact(edu.stanford.smi.protege.model.Frame)
     */
    protected Artifact createArtifact(Frame frame) {
        ProtegeArtifact art = (ProtegeArtifact) super.createArtifact(frame);       
        if (art != null) {
//	        if ((frame instanceof OWLAnonymousClass) && (!(frame instanceof OWLHasValue) || (((OWLHasValue)frame).getHasValue() instanceof RDFSClass)) && !(frame instanceof OWLCardinality) && !(frame instanceof OWLEnumeratedClass)) {
//	            art.setName("");
//	            art.setAttribute(AttributeConstants.NOM_ATTR_ARTIFACT_DISPLAY_TEXT, getBrowserText(frame));
//	        }
	        if (frame instanceof OWLAnonymousClass) {
	            art.setAttribute(AttributeConstants.NOM_ATTR_OPENABLE, new Boolean (false));
	        }
        }
        return art;
    }
	
	
	/**
	 * @see ca.uvic.csr.shrimp.jambalaya.ProtegeDataBean.ProtegeDataBean#findAllArtifactTypesInBackEnd()
	 */
	protected Vector findArtifactTypesInBackEnd() {
		Vector types = new Vector ();
		types.add(RESTRICTION_TYPE);
		types.add(LOGICAL_OPERATION_TYPE);
		types.add(PRIMITIVE_CLASS_TYPE);
		types.add(DEFINED_CLASS_TYPE);
		types.add(INDIVIDUAL_TYPE);
		types.add(ENUMERATION_CLASS_TYPE);
		types.add(RDFS_CLASS_TYPE);
		return types;
	}
	
	/**
	 * @see ca.uvic.csr.shrimp.jambalaya.ProtegeDataBean.ProtegeDataBean#findOutgoingRelationshipsInBackEnd(ca.uvic.csr.shrimp.DataBean.Artifact)
	 */
	protected Vector findOutgoingRelationshipsInBackEnd(Artifact artifact) {
        //long start = System.currentTimeMillis();
		Set rels = new HashSet (); 
	    //ProgressDialog.setNote("Finding \"" + GROUP_NAME_DOMAIN_RANGE + "\" relationships ..."); 
		findDomainRangeRels ((ProtegeArtifact)artifact, rels, true);
	    //ProgressDialog.setNote("Finding \"" + GROUP_NAME_CLASS_INSTANCE_HIERARCHY + "\" relationships ..."); 
		findOutgoingIsARelationships((ProtegeArtifact)artifact, rels);		
	    //ProgressDialog.setNote("Finding " + GROUP_NAME_PROPERTY_RESTRICTIONS + "\" relationships ..."); 
		findOutgoingConditionsRelationships((ProtegeArtifact)artifact, rels);
	    //ProgressDialog.setNote("Finding \"" + GROUP_NAME_LOGICAL_DESCRIPTION + "\" relationships ..."); 
		//if (project.getDisplayHiddenFrames()) {
		    //findOperandRelationships(artifact, rels);
		//}
	    //ProgressDialog.setNote("Finding \"" + GROUP_NAME_CONNECTING_INDIVIDUALS + "\" relationships ..."); 
		findOutgoingIndividualRelationships ((ProtegeArtifact)artifact, rels);
		
		if (UNREIFY_RELATION_INSTANCES) {
		    rels.addAll(findOutgoingUnreifiedRelations(artifact));
		}
        //long end = System.currentTimeMillis();
        //System.out.println(artifact + " findOutgoingRelationshipsInBackEnd: " + (end - start));
		return new Vector (rels);
	}
	
    /* (non-Javadoc)
     * @see ca.uvic.csr.shrimp.DataBean.ProtegeDataBean.ProtegeDataBean#findIncomingRelationshipsInBackEnd(ca.uvic.csr.shrimp.DataBean.Artifact)
     */
    protected Vector findIncomingRelationshipsInBackEnd(Artifact artifact) {
        //long start = System.currentTimeMillis();
		Set rels = new HashSet (); 
	    //ProgressDialog.setNote("Finding \"" + GROUP_NAME_DOMAIN_RANGE + "\" relationships ..."); 
		findDomainRangeRels ((ProtegeArtifact)artifact, rels, false);
	    //ProgressDialog.setNote("Finding \"" + GROUP_NAME_CLASS_INSTANCE_HIERARCHY + "\" relationships ..."); 
		findIncomingIsARelationships((ProtegeArtifact)artifact, rels);		
	    //ProgressDialog.setNote("Finding " + GROUP_NAME_PROPERTY_RESTRICTIONS + "\" relationships ..."); 
		findIncomingConditionsRelationships((ProtegeArtifact)artifact, rels);
	    //ProgressDialog.setNote("Finding \"" + GROUP_NAME_LOGICAL_DESCRIPTION + "\" relationships ..."); 
		//if (project.getDisplayHiddenFrames()) {
		    //findOperandRelationships(artifact, rels);
		//}
	    //ProgressDialog.setNote("Finding \"" + GROUP_NAME_CONNECTING_INDIVIDUALS + "\" relationships ..."); 
		findIncomingIndividualRelationships ((ProtegeArtifact)artifact, rels);
		if (UNREIFY_RELATION_INSTANCES) {
		    rels.addAll(findIncomingUnreifiedRelations(artifact));
		}
        //long end = System.currentTimeMillis();
        //System.out.println(artifact + " findIncomingRelationshipsInBackEnd: " + (end - start));
		return new Vector (rels);
    }
	
	
	private void findOutgoingIndividualRelationships (ProtegeArtifact srcArtifact, Set rels) {
		if (!(srcArtifact.getFrame() instanceof RDFIndividual)) return;
        RDFIndividual srcIndividual = (RDFIndividual) srcArtifact.getFrame();
        Collection properties = srcIndividual.getRDFProperties();
        for (Iterator iter = properties.iterator(); iter.hasNext();) {
			RDFProperty property = (RDFProperty) iter.next();
            if (shouldImportSlotAsRelationshipCheckFiltered(property, SUFFIX_CONNECTS_INDIVIDUALS)) {
                Collection propertyValues = srcIndividual.getPropertyValues(property);
                
                for (Iterator iterator = propertyValues.iterator(); iterator.hasNext();) {
                    Object propertyValue = iterator.next();
                    if (propertyValue instanceof RDFIndividual) {
                    	RDFIndividual destIndividual = (RDFIndividual) propertyValue;
                        if (shouldImportFrameAsArtifact(destIndividual)) {
                            Artifact destArtifact = findArtifact(destIndividual, true, true, true);
                            if (destArtifact != null) {
                                String relType = slotToRelationshipType(property, SUFFIX_CONNECTS_INDIVIDUALS);
        	                    Relationship rel = createRel(null, srcArtifact, destArtifact, relType);
        	                    rels.add(rel);
                            }
                        }
                    } else {
                    	// TODO should we still allow a Cls value...this is OWL-Full
                    }
                 }
            }
        }
	}
	private void findIncomingConditionsRelationships (ProtegeArtifact destArt, Set rels) {
		if (!(destArt.getFrame() instanceof RDFSNamedClass)) return;
		RDFSNamedClass destClass = (RDFSNamedClass) destArt.getFrame();
        //long start = System.currentTimeMillis();
		Collection referers = destClass.getReferringAnonymousClasses();
        //long end = System.currentTimeMillis();
        //System.out.println(destArt + " getReferringAnonymousClasses: " + (end - start));
        //System.out.println(destArt + " referrers " + referers.size());
		for (Iterator iter = referers.iterator(); iter.hasNext();) {
            RDFResource referer = (RDFResource) iter.next();
            if (referer instanceof OWLAnonymousClass) {
                if (referer instanceof OWLRestriction) {
                    OWLRestriction restriction = (OWLRestriction) referer;
                    String shortName = getBrowserText(restriction);
                    //start = System.currentTimeMillis();
                    RDFProperty property = restriction.getOnProperty();
                    //end = System.currentTimeMillis();
                    //System.out.println(destArt + " getOnProperty '" + shortName + "': " + (end - start));
                    //start = System.currentTimeMillis();
                    Collection srcClasses = restriction.getNamedSubclasses(false);
                    //end = System.currentTimeMillis();
                    //System.out.println(destArt + " getNamedSubclasses '" + shortName + "': " + (end - start));
                    for (Iterator iterator = srcClasses.iterator(); iterator.hasNext();) {
                        RDFSNamedClass srcClass = (RDFSNamedClass) iterator.next();
                        String suffix = srcClass.getSubclasses(false).contains(restriction) ? SUFFIX_NECESSARY_AND_SUFFICIENT : SUFFIX_NECESSARY;
                        if (shouldImportSlotAsRelationshipCheckFiltered(property, suffix)) {
                            ProtegeArtifact srcArt = (ProtegeArtifact) findArtifact(srcClass, true, true, true);
                            if (srcArt != null) {
                                String relType = slotToRelationshipType(property, suffix); 
                                Relationship rel = createRel(null, srcArt, destArt, relType);
                                rel.setAttribute(AttributeConstants.NOM_ATTR_REL_DISPLAY_TEXT, restrictionToRelName(srcArt, restriction, suffix));
                				rel.setAttribute(AttributeConstants.NOM_ATTR_REL_SHORT_DISPLAY_TEXT, shortName);
                				rels.add(rel);
        						//System.out.println("incoming conditions rel: " + rel);
                            }
                        }
                    }
                }
            }
        }
	}
	
	private String restrictionToRelName (ProtegeArtifact srcArt, OWLRestriction restriction, String suffix) {
        StringBuffer s = new StringBuffer();
        s.append(srcArt.getName());
        s.append(":\n   ");
        s.append(restriction.getOperator());
        s.append(" ");
        s.append(getBrowserText(restriction.getOnProperty()));
        s.append(" ");
        s.append(restriction.getFillerText());
        s.append(suffix);
        
	    return s.toString();
	}
	
	private String restrictionToShortRelName (OWLRestriction restriction) {
	    return getBrowserText(restriction.getOnProperty());
	}
	
	private void findIncomingIndividualRelationships (ProtegeArtifact destArt, Set rels) {
		if (!(destArt.getFrame() instanceof RDFIndividual)) return;
        RDFIndividual destIndiv = (RDFIndividual) destArt.getFrame();
        Collection references = destIndiv.getReferences();
        for (Iterator iter = references.iterator(); iter.hasNext();) {
            Reference reference = (Reference) iter.next();
            Frame refFrame = reference.getFrame();
            Slot refSlot = reference.getSlot();
            if (refFrame instanceof RDFIndividual && refSlot instanceof RDFProperty) {
                if (shouldImportSlotAsRelationshipCheckFiltered(refSlot, SUFFIX_CONNECTS_INDIVIDUALS)) {
	                Artifact srcArt = findArtifact(refFrame);
	                if (srcArt != null) {
						String relType = slotToRelationshipType(refSlot, SUFFIX_CONNECTS_INDIVIDUALS);
						Relationship rel = createRel (null, srcArt, destArt, relType);
						rels.add (rel);
						//System.out.println("incoming indiv rel: " + rel);
	                }
                }	                
            }
        }
	}
	
	/*
	private void findOperandRelationships (ProtegeArtifact artifact, Set rels) {
        Frame frame = artifact.getFrame();
        if (frame instanceof OWLLogicalClass) {
            OWLLogicalClass logicalCls = (OWLLogicalClass) frame;
            Collection operands = logicalCls.getOperands();
            for (Iterator iter = operands.iterator(); iter.hasNext();) {
                Frame operand = (Frame) iter.next();
                if (shouldImportFrameAsArtifact(operand, true)) {
		            Artifact child = findArtifact(frameToArtifactID(operand));
		            if (child != null) {
	                    String relType = REL_TYPE_OPERATES_ON;
	                    Relationship rel = createRel(correctID(null), artifact, child, relType);
	                    rels.add(rel);
		            }
                }
            }            
        } else if (frame instanceof OWLRestriction) {
            OWLRestriction r = (OWLRestriction) frame;
            if (r instanceof OWLCardinality) {
                // do nothing with Cardinality restrictions for now
            } else if (r instanceof OWLQuantifierRestriction) {
                // create rels from restriction to its quantifier class
                OWLQuantifierRestriction qr = (OWLQuantifierRestriction) r;
                if (!qr.isDataRestriction()) {
                    RDFSClass quantifierCls = qr.getQuantifierCls();
                    if (shouldImportFrameAsArtifact(quantifierCls, true)) {
                        Artifact child = findArtifact(frameToArtifactID(quantifierCls));
                        if (child != null) {
                            String relType = slotToRelationshipType(qr.getOnProperty()) + SUFFIX_QUANTIFIER_CLS;
                            Relationship rel = createRel(correctID(null), artifact, child, relType);
                            rels.add(rel);
                        }
                    }
                    Collection quantifierValues = qr.getQuantifierValues();
                    for (Iterator iter = quantifierValues.iterator(); iter.hasNext();) {
                        Object obj = iter.next();
                        if (!obj.equals(quantifierCls)) {
                            if (obj instanceof Frame) {
                                Frame quantifierValue = (Frame) obj;
                                if (shouldImportFrameAsArtifact(quantifierValue, true)) {
                                    Artifact child = findArtifact(frameToArtifactID(quantifierValue));
                                    if (child != null) {
                                        (new Exception ("Warning! Acceptable qantifier value other than quantifier cls: " + getBrowserText(quantifierValue))).printStackTrace();
                                        //String relType = slotToRelationshipType(qr.getOnProperty()) + SUFFIX_QUANTIFIER_VALUE;
                                        //Relationship rel = createRel(correctID(null), artifact, child, relType);
                                        //rels.add(rel);
                                    }
                                }
                            }
                        }
                    }
                } else {
                    // do nothing with datatype quantifier restrictions for now
                }
             } else if (r instanceof OWLHasValue) {
                OWLHasValue hr = (OWLHasValue) r;
                Object hasValueObj = hr.getHasValue();
                if (hasValueObj != null && hasValueObj instanceof Frame) {
                    Frame hasValueFrame = (Frame) hasValueObj;
                    if (shouldImportFrameAsArtifact(hasValueFrame, true)) {
                        Artifact child = findArtifact(frameToArtifactID(hasValueFrame));
                        if (child != null) {
    	                    String relType = slotToRelationshipType(hr.getOnProperty()) + SUFFIX_HAS_VALUE;
    	                    Relationship rel = createRel(correctID(null), artifact, child, relType);
    	                    rels.add(rel);
                        }
                    }
                }
            }
        }
	}
	*/

	/**
     * @param artifact
     * @param rels
     */
    private void findOutgoingConditionsRelationships(ProtegeArtifact srcArtifact, Set rels) {
		if (!(srcArtifact.getFrame() instanceof OWLNamedClass)) return;
        OWLNamedClass namedCls = (OWLNamedClass) srcArtifact.getFrame();
        
        //neccessary and sufficient
        Collection inheritedRestrictions = namedCls.getRestrictions(true);
        Collection necessaryRestrictions = namedCls.getRestrictions(false);
        RDFSClass definition = namedCls.getDefinition();
        if (definition != null) {
	        if (definition instanceof OWLIntersectionClass) {
	            OWLIntersectionClass intersectionCls = (OWLIntersectionClass) definition;
	            Collection operands = intersectionCls.getOperands();
	            for (Iterator iter = operands.iterator(); iter.hasNext();) {
	                RDFSClass operand = (RDFSClass) iter.next();
	                if (operand instanceof OWLRestriction) {
	                    createOutgoingRelFromRestriction(srcArtifact, (OWLRestriction)operand, rels, SUFFIX_NECESSARY_AND_SUFFICIENT);
	                    necessaryRestrictions.remove(operand);
	                    inheritedRestrictions.remove(operand);
	                }
	            }
	        } else if (definition instanceof OWLRestriction) {
	            createOutgoingRelFromRestriction(srcArtifact, (OWLRestriction)definition, rels, SUFFIX_NECESSARY_AND_SUFFICIENT);
	            necessaryRestrictions.remove(definition);
	            inheritedRestrictions.remove(definition);
	        }
        }
        
        //necessary
        for (Iterator iter = necessaryRestrictions.iterator(); iter.hasNext();) {
            OWLRestriction restriction = (OWLRestriction) iter.next();
            createOutgoingRelFromRestriction (srcArtifact, restriction, rels, SUFFIX_NECESSARY);	
            inheritedRestrictions.remove(restriction);
        }
        
        //inherited
        if (DEFAULT_IMPORT_INHERITED_RELS) {
	        for (Iterator iter = inheritedRestrictions.iterator(); iter.hasNext();) {
	            OWLRestriction restriction = (OWLRestriction) iter.next();
	            createOutgoingRelFromRestriction(srcArtifact, restriction, rels, SUFFIX_INHERITED);
	        }       
        }
    }
    
    public Collection getDomainRangeRelationships () {
        if (domainRangeRelsBuffer == null ) {
            createDomainRangeRels();
        }
        return domainRangeRelsBuffer;
    }
    
    private void createDomainRangeRels () {
        domainRangeRelsBuffer = new HashSet();
	    Collection allProperties = owlModel.getRDFProperties();
	    for (Iterator iter = allProperties.iterator(); iter.hasNext();) {
			RDFProperty property = (RDFProperty) iter.next();
			if (!shouldImportSlotAsRelationshipCheckFiltered(property, SUFFIX_DOMAIN_RANGE)) continue;
			
			// convert domain values into named classes
			List domainVals = new ArrayList (property.getDomains(true));
			List rangeDefs = new ArrayList (property.getRanges(true));
            
            // if either the domain or range is specified, but not both, then add owl:THING
			if (domainVals.isEmpty() && !rangeDefs.isEmpty()) {
			    domainVals.add(owlModel.getOWLThingClass());
            } else if (rangeDefs.isEmpty() && !domainVals.isEmpty()) {
			    rangeDefs.add(owlModel.getOWLThingClass());
			} 
			Set domains = new HashSet();
			for (Iterator iterator = domainVals.iterator(); iterator.hasNext();) {
                Frame domain = (Frame) iterator.next();
                if (domain instanceof RDFSNamedClass) {
                    domains.add(domain);
                } else if (domain instanceof OWLNAryLogicalClass) {
                    //domains.add(domain);                              
                    domains.addAll(((OWLNAryLogicalClass)domain).getNamedOperands());  // TODO is this really what should be shown?                             
                } else {
					System.err.println("domain not a RDFSNamedClass or OWLNAryLogicalClass: " + getBrowserText(domain));                  
                }
            }
			
			//convert range definitions into named classes
			Set ranges = new HashSet();
			for (Iterator rangeDefIter = rangeDefs.iterator(); rangeDefIter.hasNext();) {
	            Frame range = (Frame) rangeDefIter.next();
	            if (range instanceof RDFSNamedClass) {
	                ranges.add(range);
	            } else if (range instanceof OWLNAryLogicalClass) {
                    //ranges.add(range);                             
                    ranges.addAll(((OWLNAryLogicalClass)range).getNamedOperands());   // TODO is this really what should be shown?                            
	            } else {
					System.err.println("range not a RDFSNamedClass or OWLNAryLogicalClass: " + getBrowserText(range));
	            }
	        }
			
			// make relationships between all named classes in the domain and all named classes in the range
			for (Iterator iterator = domains.iterator(); iterator.hasNext();) {
                RDFSClass domainClass = (RDFSClass) iterator.next();
                Artifact srcArt = findArtifact(domainClass, true, false, false);
                if (srcArt != null) {
	                for (Iterator rangeIter = ranges.iterator(); rangeIter.hasNext();) {
	                    RDFSClass rangeClass = (RDFSClass) rangeIter.next();
	                    Artifact destArt = findArtifact(rangeClass, true, false, false);
	                    if (destArt != null) {
	                        String relType = slotToRelationshipType(property, SUFFIX_DOMAIN_RANGE);
	                        Relationship rel = createRel(null, srcArt, destArt, relType);
                            rel.setAttribute(AttributeConstants.NOM_ATTR_REL_SHORT_DISPLAY_TEXT, slotToRelationshipType(property, ""));
	                        domainRangeRelsBuffer.add(rel);
	                    }
	                }
                }
            }
	    }
        
    }
    

	/**
	 * Finds relationships between an artifact as a item in the domain, and the ranges of its properties
	 * @param artifact
	 * @param rels
	 */
	private void findDomainRangeRels (ProtegeArtifact artifactOfInterest, Set rels, boolean outgoing) {
	    getDomainRangeRelationships(); // ensures that domain range rels are created
	    for (Iterator iter = domainRangeRelsBuffer.iterator(); iter.hasNext();) {
            Relationship rel = (Relationship) iter.next();
            Artifact srcArt = (Artifact) rel.getArtifacts().elementAt(0);
            Artifact destArt = (Artifact) rel.getArtifacts().elementAt(1);
            if (outgoing && srcArt.equals(artifactOfInterest)) {
                rels.add(rel);
            } else if (destArt.equals(artifactOfInterest)) {
                rels.add(rel);
            }
        }
	}

    private void findOutgoingIsARelationships (ProtegeArtifact srcArtifact, Set rels) {
        if (! (srcArtifact.getFrame() instanceof RDFSNamedClass)) return;
        RDFSNamedClass srcCls = (RDFSNamedClass) srcArtifact.getFrame();
        // :DIRECT-SUBCLASSES
        Slot subClassSlot = knowledgeBase.getSlot(Model.Slot.DIRECT_SUBCLASSES);
        if (shouldImportSlotAsRelationshipCheckFiltered(subClassSlot, "")) {
		    Collection directSubclasses = srcCls.getNamedSubclasses(false);
		    for (Iterator iter = directSubclasses.iterator(); iter.hasNext();) {
	            Cls subCls = (Cls) iter.next();
	            Artifact destArtifact = findArtifact(subCls, true, true, true);
	            if (destArtifact != null) {
	                String relType = slotToRelationshipType(subClassSlot, "");
                    Relationship rel = createRel(null, srcArtifact, destArtifact, relType);
                    rels.add(rel);
	            } 
	        }
        } 
        
        // equivalent classes
        Collection equivalentClasses = srcCls.getEquivalentClasses();
        for (Iterator iter = equivalentClasses.iterator(); iter.hasNext();) {
            RDFSClass equivClass = (RDFSClass) iter.next();
            Artifact destArtifact = findArtifact(equivClass, true, true, true);
            if (destArtifact != null) {
                String relType = slotToRelationshipType(subClassSlot, "");
                Relationship rel = createRel(null, srcArtifact, destArtifact, relType);
                rels.add(rel);
            } 
        }
        
	    // :DIRECT-INSTANCES
        Slot instanceSlot = knowledgeBase.getSlot(Model.Slot.DIRECT_INSTANCES);
	    if (shouldImportSlotAsRelationshipCheckFiltered(instanceSlot, "") /*&& (CONSIDER_METACLASS_INSTANCES || !cls.isMetaCls()) */) {
		    Collection directInstances = srcCls.getInstances(false);
		    for (Iterator iter = directInstances.iterator(); iter.hasNext();) {
                Frame instance = (Frame) iter.next();
                if (instance instanceof RDFIndividual || instance instanceof RDFSClass) {
		            Artifact destArtifact = findArtifact(instance);
		            if (destArtifact != null) {
	                    String relType = slotToRelationshipType(instanceSlot, "");
	                    Relationship rel = createRel(null, srcArtifact, destArtifact, relType);
	                    rels.add(rel);
		            }
                } else {
                    //System.err.println("instance of rdfs named class is not an RDFSClass or an RDFIndividual: " + getBrowserText(instance) + ", " + instance + ", " + instance.getClass().getName());
                }
            }
	    }
 	}
    
    private void findIncomingIsARelationships (ProtegeArtifact destArt, Set rels) {
        if (! (destArt.getFrame() instanceof RDFSNamedClass)) return;
        RDFSNamedClass destClass = (RDFSNamedClass) destArt.getFrame();
        
        // :DIRECT-SUPERCLASSES
        Slot subClassSlot = knowledgeBase.getSlot(Model.Slot.DIRECT_SUBCLASSES);
        if (shouldImportSlotAsRelationshipCheckFiltered(subClassSlot, "'")) {
		    Collection directSuperclasses = destClass.getNamedSuperclasses(false);
		    for (Iterator iter = directSuperclasses.iterator(); iter.hasNext();) {
	            Cls superClass = (Cls) iter.next();
	            Artifact srcArt = findArtifact(superClass, true, true, true);
	            if (srcArt != null) {
	                String relType = slotToRelationshipType(subClassSlot, "");
                    Relationship rel = createRel(null, srcArt, destArt, relType);
                    rels.add(rel);
	            } 
	        }
        } 
        
        // equivalent classes 
        Collection equivalentClasses = destClass.getEquivalentClasses();
        for (Iterator iter = equivalentClasses.iterator(); iter.hasNext();) {
            RDFSClass equivClass = (RDFSClass) iter.next();
            Artifact srcArt = findArtifact(equivClass, true, true, true);
            if (srcArt != null) {
                String relType = slotToRelationshipType(subClassSlot, "");
                Relationship rel = createRel(null, srcArt, destArt, relType);
                rels.add(rel);
            } 
        }
        
	    // :DIRECT-TYPES
        Slot instanceSlot = knowledgeBase.getSlot(Model.Slot.DIRECT_INSTANCES);
	    if (shouldImportSlotAsRelationshipCheckFiltered(instanceSlot, "") /*&& (CONSIDER_METACLASS_INSTANCES || !cls.isMetaCls()) */) {
	        Collection types = destClass.getRDFTypes();
		    for (Iterator iter = types.iterator(); iter.hasNext();) {
                Frame type = (Frame) iter.next();
                if (type instanceof RDFSNamedClass) {
		            Artifact srcArt = findArtifact(type);
		            if (srcArt != null) {
	                    String relType = slotToRelationshipType(instanceSlot, "");
	                    Relationship rel = createRel(null, srcArt, destArt, relType);
	                    rels.add(rel);
		            }
                } else {
                    System.err.println("type of rdfs named class is not an RDFSNamedClass: " + getBrowserText(type));
                }
            }
	    }
 	}
    
    /* (non-Javadoc)
     * @see ca.uvic.csr.shrimp.DataBean.ProtegeDataBean.ProtegeDataBean#findAllArtifactsInBackEnd()
     */
    protected Vector findAllArtifactsInBackEnd() {
        Vector backEndArts = super.findAllArtifactsInBackEnd();
        if (ProgressDialog.isCancelled()) return backEndArts;
        // add anonymous classes so we can see some of the logical expressions
        
        // TODO add option to show or hide anonymous classes
//        Cls owlLogicalCls = owlModel.getCls(OWLNames.Cls.LOGICAL_CLASS);
//        Artifact owlLogicalClsArt = findArtifact(owlLogicalCls, true, false, false);
//        if (owlLogicalClsArt != null && !backEndArts.contains(owlLogicalClsArt)) {
//            backEndArts.add(owlLogicalClsArt);
//        }
//        List logicalClsSubClsesAndInstances = new ArrayList(owlLogicalCls.getSubclasses());
//        logicalClsSubClsesAndInstances.addAll(owlLogicalCls.getInstances());
//        for (Iterator iter = logicalClsSubClsesAndInstances.iterator(); iter.hasNext();) {
//            Instance logicalClsSubClsOrInstance = (Instance) iter.next();
//            Artifact anonArtifact = findArtifact(logicalClsSubClsOrInstance, true, false, false);
//            if (anonArtifact != null && !backEndArts.contains(anonArtifact)) {
//                backEndArts.add(anonArtifact);
//            }
//        }
        return backEndArts;         
    }

 
    
    /* (non-Javadoc)
     * @see ca.uvic.csr.shrimp.DataBean.ProtegeDataBean.ProtegeDataBean#findAllRelationshipTypesInBackEnd()
     */
    protected Vector findRelationshipTypesInBackEnd() {
        createGroupToRelTypeMappings();
        Vector types = new Vector (relTypeToGroupMap.keySet());
		return types;
    }
    
    /* (non-Javadoc)
	 * @see ca.uvic.csr.shrimp.DataBean.ProtegeDataBean.ProtegeDataBean#isHidden(edu.stanford.smi.protege.model.Frame)
	 */
	protected boolean isHidden(Frame frame) {
		boolean isHidden = false;
		if (!project.getDisplayHiddenFrames()) {
//            if (frame instanceof OWLIndividual) {
//                Collection rdfTypes = ((OWLIndividual)frame).getRDFTypes();
//                if (rdfTypes.size() == 1) {
//                    RDFSClass rdfType = (RDFSClass) rdfTypes.iterator().next();
//                    boolean isLogicalIndividual = rdfType.getName().equals(OWLNames.Cls.UNION_CLASS) || 
//                                                  rdfType.getName().equals(OWLNames.Cls.INTERSECTION_CLASS) || 
//                                                  rdfType.getName().equals(OWLNames.Cls.COMPLEMENT_CLASS);
//                    isHidden = isLogicalIndividual ? false : !frame.isVisible();
//                } else {
//                    isHidden = !frame.isVisible();
//                }
//            } else {
                isHidden = !frame.isVisible();
//            }
		}
		return isHidden;
	}
    
    
    /* (non-Javadoc)
     * @see ca.uvic.csr.shrimp.DataBean.ProtegeDataBean.ProtegeDataBean#isInRootClses(edu.stanford.smi.protege.model.Frame)
     */
    protected boolean isInRootClses(Frame frame) {
        boolean inRootClses = super.isInRootClses(frame) || (frame instanceof OWLAnonymousClass);
        /* TODO only show anonymous classes that are somehow related to named classes that root classes or subclasses of
        // if this is an anonymous class then see if it is the super class of some named class that is valid
        if (frame instanceof RDFSClass && ((RDFSClass)frame).isAnonymous()) {
        	OWLAnonymousClass anonymousCls = (OWLAnonymousClass) frame;
        	boolean oneSubClsIsInRootClses = false;
        	Collection subClses = anonymousCls.getDirectSubclasses();
        	for (Iterator iter = subClses.iterator(); iter.hasNext();) {
				RDFSClass subCls = (RDFSClass) iter.next();
				if (subCls instanceof OWLNamedClass) {
					oneSubClsIsInRootClses = super.isInRootClses(subCls);
					if (oneSubClsIsInRootClses) {
						break;
					}
				}
			}
            inRootClses = inRootClses || oneSubClsIsInRootClses;
       }
       */
        return inRootClses;
    }

	protected boolean slotIsRightSystemType (String relType) {
		boolean rightSystemType = super.slotIsRightSystemType(relType);
		rightSystemType = rightSystemType || /*TODO relType.equals(OWLNames.Slot.LOGICAL_OPERANDS)  ||*/ relType.equals(REL_TYPE_NECESSARY_CONDITION_OF) || relType.equals("Equivalent Class");
		return rightSystemType;
	}
	
	/* (non-Javadoc)
     * @see ca.uvic.csr.shrimp.DataBean.ProtegeDataBean.ProtegeDataBean#slotHasValidRange(edu.stanford.smi.protege.model.Slot)
     */
    protected boolean slotHasValidRange(Slot slot) {
        boolean hasValidRange = false;
        if (slot instanceof RDFProperty) {
            RDFProperty property = (RDFProperty) slot;
//            if (property instanceof OWLObjectProperty) {
//                hasValidRange = true;
//            } else if (property instanceof OWLDatatypeProperty){ 
//                hasValidRange = false;
//            } else {
//                // TODO how to handle non-OWL properties?
//                //System.err.println("Jambalaya, not sure how to handle non-OWL property: " + getBrowserText(property) + ",  " + property);
                hasValidRange = property.hasObjectRange();
//            }
        } else {
            hasValidRange = super.slotHasValidRange(slot);
        }
//        if (!hasValidRange) {
//            System.out.println("slot doesn't have valid range: " + getBrowserText(slot) + ",  " + slot);
//        }
        return hasValidRange;
    }

	protected boolean slotHasOneValidCls (Slot slot) {
		// see if there is at least one allowed class that is acceptable
		boolean hasOneValidAllowedCls = slot.getAllowedClses().isEmpty();
		for (Iterator iter = slot.getAllowedClses().iterator(); iter.hasNext() && !hasOneValidAllowedCls;) {
			Cls allowedCls = (Cls) iter.next();
			if (shouldImportFrameAsArtifact(allowedCls, true) || allowedCls instanceof OWLAnonymousClass ||  validSlotSuperClsesAndTypes.contains(allowedCls)) {
				hasOneValidAllowedCls = true;
			}
		}
		if (!hasOneValidAllowedCls) {
			return false;
		}
		
		// see if there is at least one template slot class that is acceptable
		boolean hasOneValidTemplateSlotCls = false;
		for (Iterator iter = slot.getDirectDomain().iterator(); iter.hasNext() && !hasOneValidTemplateSlotCls;) {
			Cls templateSlotCls = (Cls) iter.next();
			if (shouldImportFrameAsArtifact(templateSlotCls, true) || templateSlotCls instanceof OWLAnonymousClass || validSlotSuperClsesAndTypes.contains(templateSlotCls)) {
				hasOneValidTemplateSlotCls = true;
			}
		}
		if (!hasOneValidTemplateSlotCls) {
			return false;
		}
		return true;
	}

  
	
    /* (non-Javadoc)
     * @see ca.uvic.csr.shrimp.DataBean.ProtegeDataBean.ProtegeDataBean#getChildren(ca.uvic.csr.shrimp.DataBean.Artifact, java.lang.String[])
     */
    public Vector getChildren(Artifact artifact, String[] cprels) {
        Vector children = super.getChildren(artifact, cprels);
        /*
        // TODO don't want equivalent classes to show up as children of a class
        Frame parentFrame = ((ProtegeArtifact)artifact).getFrame();
        for (Iterator iter = ((Vector)children.clone()).iterator(); iter.hasNext();) {
            ProtegeArtifact childArtifact = (ProtegeArtifact) iter.next();
            Frame childFrame = childArtifact.getFrame();
            if (childFrame instanceof RDFSClass) {
                RDFSClass childOwlCls = (RDFSClass) childFrame;
                if (childOwlCls.getDirectSubclasses().contains(parentFrame)) {
                    children.remove(childArtifact);
                }
            }
        }
        */
        return children;
    }
    
    /* (non-Javadoc)
     * @see ca.uvic.csr.shrimp.DataBean.ProtegeDataBean.ProtegeDataBean#findRootArtifactsInBackEnd(java.lang.String[])
     */
    protected Vector findRootArtifactsInBackEnd(String[] cprels) {
		if (CollectionUtils.haveSameElements(cprels, SUBCLASS_OF_AND_INSTANCE_OF_HIERARCHY) || 
		        CollectionUtils.haveSameElements(cprels, SUBCLASS_OF_HIERARCHY) ||
		        CollectionUtils.haveSameElements(cprels, INSTANCE_OF_HIERARCHY)) {
            Vector roots = super.findRootArtifactsInBackEnd(cprels);
//            Cls owlLogicalCls = owlModel.getCls(OWLNames.Cls.LOGICAL_CLASS);
//            Artifact owlLogicalClsArt = findArtifact(owlLogicalCls, true, false, false);
//            if (owlLogicalClsArt != null) {
//                roots.add(owlLogicalClsArt);
//            }
            return roots;
		} else if (cprels.length == 0) {
			return super.findRootArtifactsInBackEnd(cprels);
		} else {
		    return defaultFindRootArtifactsInBackEnd(cprels);
		}
	}

	/**
	 * Creates a relationship from the given artifact to a destination based on the 
	 * given restriction. The new relationship is added to the passed in Vector.
	 * @param srcArt
	 * @param r
	 * @param rels
	 */
	private void createOutgoingRelFromRestriction (ProtegeArtifact srcArt, OWLRestriction r, Set rels, String suffix) {
        if (!shouldImportSlotAsRelationshipCheckFiltered(r.getOnProperty(), suffix)) return;
	    Cls destCls = null;
		if (r instanceof OWLQuantifierRestriction) {
		    OWLQuantifierRestriction qr = (OWLQuantifierRestriction)r;
		    RDFResource filler = qr.getFiller();
		    if (filler instanceof RDFSClass) {
				destCls = (RDFSClass) filler;	
		    } else {
		        //System.err.println("filler of quantifier restriction is not an RDFSClass: " + filler);
		    }
		} else if (r instanceof OWLHasValue) {
		    OWLHasValue hr = (OWLHasValue)r;
		    Object hasValue = hr.getHasValue();
		    if (hasValue instanceof RDFSClass) {
		        destCls = (RDFSClass) hasValue;
		    } else {
		    	//System.err.println("hasValue of hasValue restriction is not an RDFSClass: " + hasValue);
		    }
		}
		if (destCls != null) {
			Artifact destArt = findArtifact(destCls);
			if (destArt != null) {
				String relType = slotToRelationshipType(r.getOnProperty(), suffix);
				Relationship rel = createRel(null, srcArt, destArt, relType);
				rel.setAttribute(AttributeConstants.NOM_ATTR_REL_DISPLAY_TEXT, restrictionToRelName(srcArt, r, suffix));
				rel.setAttribute(AttributeConstants.NOM_ATTR_REL_SHORT_DISPLAY_TEXT, restrictionToShortRelName(r));
				rels.add(rel);
                //System.out.println("outgoing conditions rel: " + rel);
			}
		}
	}
	
        
    /* (non-Javadoc)
     * @see ca.uvic.csr.shrimp.DataBean.ProtegeDataBean.ProtegeDataBean#createGroupToRelTypeMappings()
     */
    protected void createGroupToRelTypeMappings() {
        relTypeToGroupMap.clear();
        
        //add "is-a" rel types
        relTypeToGroupMap.put(DIRECT_INSTANCE_SLOT_TYPE, GROUP_NAME_CLASS_INSTANCE_HIERARCHY);
        relTypeToGroupMap.put(DIRECT_SUBCLASS_SLOT_TYPE, GROUP_NAME_CLASS_INSTANCE_HIERARCHY);
        
        // add "logical operand" rel types
        //if (project.getDisplayHiddenFrames()) {
        	// relTypeToGroupMap.put(REL_TYPE_OPERATES_ON, GROUP_NAME_LOGICAL_DESCRIPTION);
        //}
        
        //relTypeToGroupMap.put(REL_TYPE_NECESSARY_CONDITION_OF, GROUP_NAME_LOGICAL_DESCRIPTION);
        //relTypeToGroupMap.put(REL_TYPE_EQUIVALENT_TO, GROUP_NAME_LOGICAL_DESCRIPTION);
        
        // add rdf, rdfs, and owl rel types
        List rdfOwlSlots = new ArrayList (RDF_RDFS_OWL_SLOT_NAMES.length);
        for (int i = 0; i < RDF_RDFS_OWL_SLOT_NAMES.length; i++) {
            String slotName = RDF_RDFS_OWL_SLOT_NAMES[i];
            Slot rdfOwlSlot = knowledgeBase.getSlot(slotName);
            if (rdfOwlSlot != null) {
                rdfOwlSlots.add(rdfOwlSlot);
                if (shouldImportSlotAsRelationship(rdfOwlSlot)) {
                    relTypeToGroupMap.put(slotToRelationshipType(rdfOwlSlot, ""), GROUP_NAME_SYSTEM);
                }
            }
        }
        
        // add all other rel types
        // TODO is there a quick way to see if a particular rel type will never show up on screen
        Collection slots = knowledgeBase.getSlots();
        for (Iterator iter = slots.iterator(); iter.hasNext();) {
            Slot slot = (Slot) iter.next();
            if (shouldImportSlotAsRelationship(slot) &&!rdfOwlSlots.contains(slot)) {
	            String baseRelType = slotToRelationshipType(slot, "");
	            if (baseRelType.equals(DIRECT_INSTANCE_SLOT_TYPE) || baseRelType.equals(DIRECT_SUBCLASS_SLOT_TYPE)) {
	                continue;
	            }
	            if (slot.isSystem()) {
	                relTypeToGroupMap.put(baseRelType, GROUP_NAME_SYSTEM);
	            } else {
		            relTypeToGroupMap.put(baseRelType + SUFFIX_CONNECTS_INDIVIDUALS, GROUP_NAME_CONNECTING_INDIVIDUALS);
		            relTypeToGroupMap.put(baseRelType + SUFFIX_NECESSARY, GROUP_NAME_PROPERTY_RESTRICTIONS);
		            relTypeToGroupMap.put(baseRelType + SUFFIX_NECESSARY_AND_SUFFICIENT, GROUP_NAME_PROPERTY_RESTRICTIONS);
		            relTypeToGroupMap.put(baseRelType + SUFFIX_DOMAIN_RANGE, GROUP_NAME_DOMAIN_RANGE);
		            if (DEFAULT_IMPORT_INHERITED_RELS) {
		                relTypeToGroupMap.put(baseRelType + SUFFIX_INHERITED, GROUP_NAME_INHERITED);
		            }
		            //relTypeToGroupMap.put(baseRelType + SUFFIX_HAS_VALUE, GROUP_NAME_LOGICAL_DESCRIPTION);
		            //relTypeToGroupMap.put(baseRelType + SUFFIX_QUANTIFIER_CLS, GROUP_NAME_LOGICAL_DESCRIPTION);
		            //relTypeToGroupMap.put(baseRelType + SUFFIX_QUANTIFIER_VALUE, GROUP_NAME_CONNECTING_INDIVIDUALS);
	            }
            } //else {
                //System.out.println("not importing slot: " + getBrowserText(slot));
            //}
        }
        
        createGroupToRelTypeMappingsUnreified();
    }

    public Icon getArtifactIcon(Artifact artifact) {
    	Icon icon = super.getArtifactIcon(artifact);
        // OverlayIcons appear at (0, 0) when paintIcon is called with y > 0
        if (icon instanceof OverlayIcon) {
            icon = new FixedOverlayIcon((OverlayIcon)icon);
        }
        return icon;
    }
        
    /**
     * Wrapper for OverlayIcon objects to prevent the icon from being drawn at (0, 0).
     * 
     * @author ccallendar
     */
    class FixedOverlayIcon implements Icon {

        private OverlayIcon icon = null;

        public FixedOverlayIcon(OverlayIcon icon) {
            this.icon = icon;
        }


        public int getIconHeight() {
            return icon.getIconHeight();
        }


        public int getIconWidth() {
            return icon.getIconWidth();
        }

        /**
         * Calls {@link OverlayIcon#paintIcon(java.awt.Component, java.awt.Graphics, int, int)}
         * with a y value of zero to prevent that method from setting x and y to zero. 
         */
        public void paintIcon(Component c, Graphics g, int x, int y) {
            icon.paintIcon(c, g, x, 0);
        }
    
    }


}
